#!/usr/bin/env python2
# -*- coding: utf-8 -*-

# jp: a SAT command line tool
# Copyright (C) 2009-2018 Jérôme Poisson (goffi@goffi.org)

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from sat_frontends.jp import base
from sat_frontends.jp.constants import Const as C
from sat_frontends.tools import jid
from sat.core.i18n import _
from sat.tools.utils import clean_ustr
from sat.tools.common import data_format
from sat.tools.common.ansi import ANSI as A
from functools import partial

__commands__ = ["Message"]


class Send(base.CommandBase):
    def __init__(self, host):
        super(Send, self).__init__(host, "send", help=_("send a message to a contact"))

    def add_parser_options(self):
        self.parser.add_argument(
            "-l", "--lang", type=str, default="", help=_(u"language of the message")
        )
        self.parser.add_argument(
            "-s",
            "--separate",
            action="store_true",
            help=_(
                u"separate xmpp messages: send one message per line instead of one "
                u"message alone."
            ),
        )
        self.parser.add_argument(
            "-n",
            "--new-line",
            action="store_true",
            help=_(
                u"add a new line at the beginning of the input (usefull for ascii art ;))"
            ),
        )
        self.parser.add_argument(
            "-S",
            "--subject",
            type=base.unicode_decoder,
            help=_(u"subject of the message"),
        )
        self.parser.add_argument(
            "-L", "--subject_lang", type=str, default="", help=_(u"language of subject")
        )
        self.parser.add_argument(
            "-t",
            "--type",
            choices=C.MESS_TYPE_STANDARD + (C.MESS_TYPE_AUTO,),
            default=C.MESS_TYPE_AUTO,
            help=_("type of the message"),
        )
        syntax = self.parser.add_mutually_exclusive_group()
        syntax.add_argument("-x", "--xhtml", action="store_true", help=_(u"XHTML body"))
        syntax.add_argument("-r", "--rich", action="store_true", help=_(u"rich body"))
        self.parser.add_argument(
            "jid", type=base.unicode_decoder, help=_(u"the destination jid")
        )

    def start(self):
        if self.args.xhtml and self.args.separate:
            self.disp(
                u"argument -s/--separate is not compatible yet with argument -x/--xhtml",
                error=True,
            )
            self.host.quit(2)

        jids = self.host.check_jids([self.args.jid])
        jid = jids[0]
        self.sendStdin(jid)

    def sendStdin(self, dest_jid):
        """Send incomming data on stdin to jabber contact

        @param dest_jid: destination jid
        """
        header = "\n" if self.args.new_line else ""
        stdin_lines = [
            stream.decode("utf-8", "ignore") for stream in sys.stdin.readlines()
        ]
        extra = {}
        if self.args.subject is None:
            subject = {}
        else:
            subject = {self.args.subject_lang: self.args.subject}

        if self.args.xhtml or self.args.rich:
            key = u"xhtml" if self.args.xhtml else u"rich"
            if self.args.lang:
                key = u"{}_{}".format(key, self.args.lang)
            extra[key] = clean_ustr(u"".join(stdin_lines))
            stdin_lines = []

        if self.args.separate:  # we send stdin in several messages
            if header:
                self.host.bridge.messageSend(
                    dest_jid,
                    {self.args.lang: header},
                    subject,
                    self.args.type,
                    profile_key=self.profile,
                    callback=lambda: None,
                    errback=lambda ignore: ignore,
                )

            for line in stdin_lines:
                self.host.bridge.messageSend(
                    dest_jid,
                    {self.args.lang: line.replace("\n", "")},
                    subject,
                    self.args.type,
                    extra,
                    profile_key=self.host.profile,
                    callback=lambda: None,
                    errback=lambda ignore: ignore,
                )

        else:
            msg = (
                {self.args.lang: header + clean_ustr(u"".join(stdin_lines))}
                if not (self.args.xhtml or self.args.rich)
                else {}
            )
            self.host.bridge.messageSend(
                dest_jid,
                msg,
                subject,
                self.args.type,
                extra,
                profile_key=self.host.profile,
                callback=lambda: None,
                errback=lambda ignore: ignore,
            )


class MAM(base.CommandBase):

    def __init__(self, host):
        super(MAM, self).__init__(
            host, "mam", use_output=C.OUTPUT_MESS, use_verbose=True, help=_(u"query archives using MAM"))
        self.need_loop=True

    def add_parser_options(self):
        self.parser.add_argument(
            "-s", "--service", type=base.unicode_decoder, default=u"",
            help=_(u"jid of the service (default: profile's server"))
        self.parser.add_argument(
            "-S", "--start", dest="mam_start", type=base.date_decoder,
            help=_(u"start fetching archive from this date (default: 1 day ago)"))
        self.parser.add_argument(
            "-E", "--end", dest="mam_end", type=base.date_decoder,
            help=_(u"end fetching archive after this date (default: no limit)"))
        self.parser.add_argument(
            "-W", "--with", dest="mam_with", type=base.unicode_decoder,
            help=_(u"retrieve only archives with this jid"))
        self.parser.add_argument(
            "-M", "--max", dest="rsm_max", type=int, default=20,
            help=_(u"maximum number of items to retrieve, using RSM (default: 20))"))

    def _sessionInfosGetCb(self, session_info, data, metadata):
        self.host.own_jid = jid.JID(session_info[u"jid"])
        self.output(data)
        # FIXME: metadata are not displayed correctly and don't play nice with output
        #        they should be added to output data somehow
        if self.verbosity:
            for value in (u"rsm_first", u"rsm_last", u"rsm_index", u"rsm_count",
                          u"mam_complete", u"mam_stable"):
                if value in metadata:
                    label = value.split(u"_")[1]
                    self.disp(A.color(
                        C.A_HEADER, label, u': ' , A.RESET, metadata[value]))

        self.host.quit()

    def _MAMGetCb(self, result):
        data, metadata, profile = result
        self.host.bridge.sessionInfosGet(self.profile,
            callback=partial(self._sessionInfosGetCb, data=data, metadata=metadata),
            errback=self.errback)

    def start(self):
        if self.args.mam_start is None:
            self.args.mam_start = base.date_decoder(u"-1 day")
        extra = {
            u"mam_start": float(self.args.mam_start),
        }
        if self.args.mam_end is not None:
            extra[u"mam_end"] = float(self.args.mam_end)
        if self.args.mam_with is not None:
            extra[u"mam_with"] = self.args.mam_with
        if self.args.rsm_max is not None:
            extra[u"rsm_max"] = self.args.rsm_max
        self.host.bridge.MAMGet(
            self.args.service, data_format.serialise(extra), self.profile,
            callback=self._MAMGetCb, errback=self.errback)


class EncryptionAlgorithms(base.CommandBase):

    def __init__(self, host):
        extra_outputs = {"default": self.default_output}
        super(EncryptionAlgorithms, self).__init__(
            host, "algorithms",
            use_output=C.OUTPUT_LIST_DICT,
            extra_outputs=extra_outputs,
            use_profile=False,
            help=_("show available encryption algorithms"))
        self.need_loop = True

    def add_parser_options(self):
        pass

    def encryptionPluginsGetCb(self, plugins):
        self.output(plugins)
        self.host.quit()

    def default_output(self, plugins):
        if not plugins:
            self.disp(_(u"No encryption plugin registered!"))
            self.host.quit(C.EXIT_NOT_FOUND)
        else:
            self.disp(_(u"Following encryption algorithms are available: {algos}").format(
                algos=', '.join([p['name'] for p in plugins])))
            self.host.quit()

    def start(self):
        self.host.bridge.encryptionPluginsGet(
            callback=self.encryptionPluginsGetCb,
            errback=partial(
                self.errback,
                msg=_(u"can't retrieve plugins: {}"),
                exit_code=C.EXIT_BRIDGE_ERRBACK,
            ),
        )


class Encryption(base.CommandBase):
    subcommands = (EncryptionAlgorithms,)

    def __init__(self, host):
        super(Encryption, self).__init__(
            host, "encryption", use_profile=False, help=_("encryption sessions handling")
        )


class Message(base.CommandBase):
    subcommands = (Send, MAM, Encryption)

    def __init__(self, host):
        super(Message, self).__init__(
            host, "message", use_profile=False, help=_("messages handling")
        )
